/*
** Copyright (C) 2016 by VisTracks, Inc
**
** THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF  VisTracks, Inc.
** The copyright notice above does not evidence any actual
** or intended publication of such source code.
*/

package com.vistracks.samplebluetoothmanager;

import android.app.Activity;
import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothManager;
import android.bluetooth.BluetoothSocket;
import android.content.Intent;
import android.os.Bundle;
import android.os.CountDownTimer;
import android.os.Handler;
import android.util.Log;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import android.bluetooth.BluetoothSocket;
import android.os.Handler;
import android.util.Log;
import android.view.View;
import android.widget.TextView;

import org.joda.time.DateTime;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;


public class ATrackAx9BluetoothManager extends Activity{
    private static final String TAG = ATrackAx9BluetoothManager.class.getSimpleName();
    private static final String MANAGER_NAME = "ATrackAx9";
    private String device;
    private BluetoothDevice btDevice;
    private final static String VBUS_ATRACK_ODB2_SPEED_KPH = "VBUS_ATRACK_ODB2_SPEED_KPH";

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_display_data);
        this.device = getIntent().getStringExtra("connector");
        this.btDevice = getIntent().getParcelableExtra("device");

        startBluetoothConnection(btDevice);
    }

    protected void startBluetoothConnection(BluetoothDevice btDevice) {
        Log.d(TAG, "Starting Bluetooth connection..");
        try {
            // Instantiate a BluetoothSocket for the remote device and connect it.
            BluetoothSocket btSock = btDevice.createRfcommSocketToServiceRecord(UUID.fromString("00001101-0000-1000-8000-00805F9B34FB"));
            btSock.connect();
            createDataReader(btSock);
        } catch (IOException e) {
            Log.e(TAG, "Bluetooth connection failed.", e);
        }
    }

    public AtrackAx9SocketReader createDataReader(BluetoothSocket btSock) {
        return new AtrackAx9SocketReader(device, btSock, this);
    }

    private static class AtrackAx9SocketReader {
        private String device;
        BluetoothSocket btSock;
        private ATrackAx9BluetoothManager manager;

        AtrackAx9SocketReader(String device, BluetoothSocket btSock, ATrackAx9BluetoothManager manager) {
            this.device = device;
            this.btSock = btSock;
            this.manager = manager;

            new CountDownTimer(5000, 1000) {
                VbusData vbusData = new VbusData();

                public void onTick(long millisUntilFinished) {
                }

                public void onFinish() {
                    try {
                        vbusData = new VbusData();
                        populateVbus(vbusData);
                    } catch (IOException e) {
                        Log.e(TAG, "populateVbusFailed", e);
                    }
                    this.start();
                }
            }.start();
        }


        protected void populateVbus(VbusData vbusData) throws IOException {
            if (!btSock.isConnected())
                return;

            // Default timestamp is now
            vbusData.setTimestamp(DateTime.now());
            Map<String, String> values = receiveData();

            if (values.get(VbusData.VBUS_ENG_RPM) != null) {
                Double engineRPM = parseAndScaleEngineSpeed(values.get(VbusData.VBUS_ENG_RPM));
                vbusData.setEngRpm(engineRPM);
                vbusData.setEngineRpmTimestamp(DateTime.now());
                manager.writeDataToView("VBUS_ENG_RPM", engineRPM.toString());
            }
            if (values.get(VbusData.VBUS_ENG_TOTAL_HOURS) != null) {
                Double engTotalHours = parseAndScaleEngineHours(values.get(VbusData.VBUS_ENG_TOTAL_HOURS));
                vbusData.setEngineTotalHours(engTotalHours);
                vbusData.setEngineTotalHoursTimestamp(DateTime.now());
                manager.writeDataToView("VBUS_ENG_TOTAL_HOURS", engTotalHours.toString());
            }
            if (values.get(VbusData.VBUS_LATITUDE) != null) {
                Double latitude = parseAndScaleLatOrLon(values.get(VbusData.VBUS_LATITUDE));
                if (latitude != 0) {
                    vbusData.setLatitude(latitude);
                    vbusData.setLatlonTimestamp(DateTime.now());
                    manager.writeDataToView("VBUS_LATITUDE", latitude.toString());
                }
            }
            if (values.get(VbusData.VBUS_LONGITUDE) != null) {
                Double longitude = parseAndScaleLatOrLon(values.get(VbusData.VBUS_LONGITUDE));
                if (longitude != 0) {
                    vbusData.setLongitude(longitude);
                    vbusData.setLatlonTimestamp(DateTime.now());
                    manager.writeDataToView("VBUS_LONGITUDE", longitude.toString());
                }
            }
            if (values.get(VbusData.VBUS_ODOMETER_KM) != null) {
                Double odometerKm = Double.parseDouble(values.get(VbusData.VBUS_ODOMETER_KM));
                vbusData.setOdometerKm(odometerKm);
                vbusData.setOdometerKmTimestamp(DateTime.now());
                manager.writeDataToView("VBUS_ODOMETER_KM", odometerKm.toString());
            }
            if (values.get(VbusData.VBUS_VEHICLE_SPEED_KPH) != null) {
                // J1939 Vehicle Speed in CPS
                Double vehicleSpeed = parseAndScaleVehicleSpeed(values.get(VbusData.VBUS_VEHICLE_SPEED_KPH));
                vbusData.setVehicleSpeedKph(vehicleSpeed);
                vbusData.setVehicleSpeedKphTimestamp(DateTime.now());
                manager.writeDataToView("VBUS_VEHICLE_SPEED_KPH", vehicleSpeed.toString());
            }
            if (values.get(VBUS_ATRACK_ODB2_SPEED_KPH) != null) {
                // ODB2 speed is in different units than J1939 Speed
                // For ODB2 the units are km/hour
                Double vehicleSpeed = Double.parseDouble(values.get(VBUS_ATRACK_ODB2_SPEED_KPH));
                vbusData.setVehicleSpeedKph(vehicleSpeed);
                vbusData.setVehicleSpeedKphTimestamp(DateTime.now());
                manager.writeDataToView("VBUS_VEHICLE_SPEED_KPH", vehicleSpeed.toString());
            }
            if (values.get(VbusData.VBUS_TIMESTAMP) != null) {
                Long l = Long.parseLong(values.get(VbusData.VBUS_TIMESTAMP));
                // Multiply by 1000 to convert unix to joda time
                DateTime d = new DateTime(l * 1000);
                vbusData.setTimestamp(d);
                manager.writeDataToView("VBUS_TIMESTAMP", d.toString());
            }
            if (values.get(VbusData.VBUS_VIN) != null) {
                vbusData.setVin(values.get(VbusData.VBUS_VIN));
                manager.writeDataToView("VBUS_VIN", values.get(VbusData.VBUS_VIN).toString());
            }
            if (values.get(VbusData.VBUS_TRANS_GEAR) != null) {
                vbusData.setTransCurrentGear(Integer.valueOf(values.get(VbusData.VBUS_TRANS_GEAR)));
            }
            if (values.get(VbusData.VBUS_FUEL_USED) != null) {
                Double fuelUsedG = parseAndScaleFuelUsed(values.get(VbusData.VBUS_FUEL_USED));
                vbusData.setFuelUsed(fuelUsedG);
            }
            Log.d(TAG, "Completed VbusData population");
        }

        private Map<String, String> receiveData() throws IOException {
            Map<String, String> retVals = new HashMap<>();
            InputStream in = btSock.getInputStream();
            StringBuilder res = new StringBuilder();
            while (true) {
                byte b = (byte) in.read();
                if (b == -1) {// -1 if the end of the stream is reached
                    break;
                }
                char c = (char) b;
                if (c == '\n') {// read until '\n' arrives
                    break;
                }
                res.append(c);
            }
            String[] vbusRaw = res.toString().split(",[ ]*");

            //if the data is valid in J1939 format, parse it
            if (vbusRaw.length == 28) {
                retVals.put(VbusData.VBUS_TIMESTAMP, vbusRaw[6]);
                retVals.put(VbusData.VBUS_LONGITUDE, vbusRaw[8]);
                retVals.put(VbusData.VBUS_LATITUDE, vbusRaw[9]);
                retVals.put(VbusData.VBUS_HEADING, vbusRaw[10]);
                retVals.put(VbusData.VBUS_ENG_RPM, vbusRaw[22]);
                retVals.put(VbusData.VBUS_VEHICLE_SPEED_KPH, vbusRaw[23]);
                retVals.put(VbusData.VBUS_ODOMETER_KM, vbusRaw[24]);
                retVals.put(VbusData.VBUS_ENG_TOTAL_HOURS, vbusRaw[25]);
                retVals.put(VbusData.VBUS_VIN, vbusRaw[26]);
                //trim off quirky characters attached to vin
                if (vbusRaw[26].length() > 17) {
                    String vin = vbusRaw[26].substring(0, 17);
                    retVals.put(VbusData.VBUS_VIN, vin);
                }
                retVals.put(VbusData.VBUS_FUEL_USED, vbusRaw[27]);
            }
            //if the data is valid in OBDII format, parse it
            else if(vbusRaw.length == 25){
                retVals.put(VbusData.VBUS_TIMESTAMP, vbusRaw[6]);
                retVals.put(VbusData.VBUS_LONGITUDE, vbusRaw[8]);
                retVals.put(VbusData.VBUS_LATITUDE, vbusRaw[9]);
                retVals.put(VbusData.VBUS_HEADING, vbusRaw[10]);
                retVals.put(VbusData.VBUS_ODOMETER_KM, vbusRaw[12]);

                // ODB2 speed is in different units than J1939 Speed
                retVals.put(VBUS_ATRACK_ODB2_SPEED_KPH, vbusRaw[15]);

                retVals.put(VbusData.VBUS_ENG_RPM, vbusRaw[22]);
                retVals.put(VbusData.VBUS_VIN, vbusRaw[23]);
                if (vbusRaw[23].length() > 17) {
                    String vin = vbusRaw[23].substring(0, 17);
                    retVals.put(VbusData.VBUS_VIN, vin);
                }
                retVals.put(VbusData.VBUS_ENG_TOTAL_HOURS, vbusRaw[24]);
            }
            else if (vbusRaw[0].compareTo("AX9 Bluetooth Connected\r") == 0) {
                configureDevice();
            }
            return retVals;
        }

        private void configureDevice() {
            List<String> configurationCommands = new ArrayList<>();

            if (device.equals("J1939")) {
                configurationCommands.add("AT$REST=14,7\r\n");
                configurationCommands.add("AT$OBDS=19\r\n");
                configurationCommands.add("AT$FMSC=1,\"%JH2%JH1%JL3%JL2%JS1%JL4\"\r\n");
                configurationCommands.add("AT$TRAC=,,,,,,18\r\n");
            } else if (device.equals("OBDII")) {
                configurationCommands.add("AT$REST=14,7\r\n");
                configurationCommands.add("AT$OBDS=0\r\n");
                configurationCommands.add("AT$FORM=,,,\"%RP%VN%EH\"\r\n");
                configurationCommands.add("AT$TRAC=,,,,,,18\r\n");
            }

            for (String command : configurationCommands) {
                byte[] bytes = command.getBytes();
                try {
                    btSock.getOutputStream().write(bytes);
                    btSock.getOutputStream().flush();
                } catch (IOException e) {
                    Log.e(TAG, "Error configuring Atrack", e);
                }
            }
        }

        private Double parseAndScaleFuelUsed(String s) {
            Double fuelUsedMl = Double.valueOf(s);
            Double fuelUsedG = fuelUsedMl * 0.001;
            return fuelUsedG;
        }

        private Double parseAndScaleEngineCoolantTemp(String s) {
            Double engCoolantTempSixteenthC = Double.valueOf(s);
            Double engCoolantTempC = engCoolantTempSixteenthC / 16.0;
            return engCoolantTempC;
        }

        private Double parseAndScaleEngineHours(String s) {
            Double engTwentiethHours = Double.valueOf(s);
            Double engHours = engTwentiethHours * 0.05;
            return engHours;
        }

        private Double parseAndScaleLatOrLon(String s) {
            double latOrLon = Double.valueOf(s);
            return latOrLon * 0.000001;
        }
        private Double parseAndScaleVehicleSpeed(String s) {
            Double speedCPS = Double.valueOf(s);
            Double speedKPH = speedCPS / 256;
            return speedKPH;
        }

        private Double parseAndScaleEngineSpeed(String s) {
            Double engSpeedRPM = Double.valueOf(s);
            engSpeedRPM = engSpeedRPM * 0.125;
            return engSpeedRPM;
        }
    }

    private void writeDataToView(String field, String value){
        int id = getResources().getIdentifier(field, "id", getApplicationContext().getPackageName());
        TextView view = (TextView) findViewById(id);
        view.setText(field+": "+value);
    }
}