/*
** Copyright (C) 2017 by VisTracks, Inc
**
** THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF  VisTracks, Inc.
** The copyright notice above does not evidence any actual
** or intended publication of such source code.
*/


package com.vistracks.samplebluetoothmanager;

import org.joda.time.DateTime;

import java.util.HashMap;
import java.util.Map;


public class VbusData {

    public final static String VBUS_BATT_VOLTAGE = "VBUS_BATT_VOLTAGE";
    public final static String VBUS_DTC = "VBUS_DTC";
    public final static String VBUS_DEVICE_SERIAL_NUMBER = "VBUS_DEVICE_SERIAL_NUMBER";
    public final static String VBUS_ENG_COOLANT_TEMP = "VBUS_ENG_COOLANT_TEMP";
    public final static String VBUS_ENG_OIL_PRESSURE = "VBUS_ENG_OIL_PRESSURE";
    public final static String VBUS_ENG_RPM = "VBUS_ENG_RPM";
    public final static String VBUS_ENG_RPM_TIMESTAMP = "VBUS_ENG_RPM_TIMESTAMP";
    public final static String VBUS_ENG_TOTAL_HOURS = "VBUS_ENG_TOTAL_HOURS";
    public final static String VBUS_ENG_TOTAL_HOURS_TIMESTAMP = "VBUS_ENG_TOTAL_HOURS_TIMESTAMP";
    public final static String VBUS_FUEL_ECONOMY = "VBUS_FUEL_ECONOMY";
    public final static String VBUS_FUEL_LEVEL = "VBUS_FUEL_LEVEL";
    public final static String VBUS_FUEL_RATE = "VBUS_FUEL_RATE";
    public final static String VBUS_FUEL_USED = "VBUS_FUEL_USED";
    public final static String VBUS_HEADING = "VBUS_HEADING";
    public final static String VBUS_HI_RES_ODOMETER = "VBUS_HI_RES_ODOMETER";
    public final static String VBUS_HI_RES_ODOMETER_TIMESTAMP = "VBUS_HI_RES_ODOMETER_TIMESTAMP";
    public final static String VBUS_HI_REZ_TRIP = "VBUS_HI_REZ_TRIP";
    public final static String VBUS_IGNITION = "VBUS_IGNITION";
    public final static String VBUS_LATITUDE = "VBUS_LATITUDE";
    public final static String VBUS_LONGITUDE = "VBUS_LONGITUDE";
    public final static String VBUS_MANAGER = "VBUS_MANAGER";
    public final static String VBUS_ODOMETER_KM = "VBUS_ODOMETER_KM";
    public final static String VBUS_ODOMETER_KM_TIMESTAMP = "VBUS_ODOMETER_KM_TIMESTAMP";
    public final static String VBUS_PARKING_BRAKE = "VBUS_PARKING_BRAKE";
    public final static String VBUS_PTO_STATUS = "VBUS_PTO_STATUS";
    public final static String VBUS_SEATBELT = "VBUS_SEATBELT";
    public final static String VBUS_THROTTLE_POSITION = "VBUS_THROTTLE_POSITION";
    public final static String VBUS_TIMESTAMP = "VBUS_TIMESTAMP";
    public final static String VBUS_TRANS_GEAR = "VBUS_TRANS_GEAR";
    public final static String VBUS_TRIP_ENGINE_HOURS = "VBUS_TRIP_ENGINE_HOURS";
    public final static String VBUS_TRIP_FUEL_USED = "VBUS_TRIP_FUEL_USED";
    public final static String VBUS_TRIP_ODOMETER = "VBUS_TRIP_ODOMETER";
    public final static String VBUS_TURN_SIGNAL = "VBUS_TURN_SIGNAL";
    public final static String VBUS_VEHICLE_SPEED_KPH = "VBUS_VEHICLE_SPEED_KPH";
    public final static String VBUS_VEHICLE_SPEED_KPH_TIMESTAMP = "VBUS_VEHICLE_SPEED_KPH_TIMESTAMP";
    public final static String VBUS_VIN = "VBUS_VIN";

    private transient double ambient;       // Geometris only
    private Long assetId;
    private long userId;
    private Double batteryVoltage;      // battery voltage
    private transient String deviceSerialNumber;
    private Long driverHistoryId;
    private transient String dtc;                 // DTC Code
    private transient DateTime dtcTimestamp;
    private transient DateTime engineTotalHoursTimestamp;
    private Double engineSpeedRpm;              // EngineSpeed
    private Double engineCoolantTemperature;    // EngineCoolantTemp in Celsius
    private Double engineOilPressure;           // EngineOilPressure
    private Double engineTotalHours;            // TotalEngineHours
    private transient DateTime engineRpmTimestamp;
    private Double fuelEconomy;                 // FuelEconomy - liters/hour
    private transient Double fuelLevel;         // FuelLevel
    private Double fuelRate;                    // FuelRate
    private Double totalFuelUsed;               // TotalFuelUsed
    private transient Double heading;           // GPS heading
    private Double highResOdometer;             // High Res Odometer in kilometer (km)
    private transient DateTime highResOdometerTimestamp;
    private transient Double highResTripKm;     // HiResTotalDistance
    private Boolean ignition;
    private Double latitude;
    private Double longitude;
    private transient DateTime latlonTimestamp;
    private transient String managerName;
    private transient double milStatus;
    private Double odometer;            // TotalDistance in kilometer (km)
    private transient DateTime odometerKmTimestamp;
    private Integer parkingBrake;
    private Boolean ptoStatus;          // Power take off
    private Boolean seatBelt;           // SeatBelt
    private Double throttle;            // Throttle
    private DateTime timestamp;
    private transient Double tripEngineHours;
    private transient Double tripFuelUsed;
    private Double tripOdometer;        // TripDistance in kilometer (km)
    private transient Integer turnSignal;
    private transient DateTime vehicleSpeedKphTimestamp;
    private Integer transCurrentGear;   // TransCurrentGear
    private Double vehicleSpeed;        // VehicleSpeed in kilometer per hour (kph)
    private String vin;

    private transient Map<String, String> fields = new HashMap<>();

    public VbusData() {
        managerName = "";
    }

    public Map<String, String> getFields() {
        return fields;
    }

    public VbusData putAllFields(Map<String, String> fields) {
        this.fields.putAll(fields);
        return this;
    }

    public VbusData putField(String name, String value) {
        fields.put(name, value);
        return this;
    }

    public String getField(String fieldName) {
        return fields.get(fieldName);
    }

    public Double getAmbient() {
        return ambient;
    }

    public VbusData setAmbient(Double ambient) {
        this.ambient = ambient;
        return this;
    }

    public Double getBatteryVoltage() {
        return batteryVoltage;
    }

    public VbusData setBatteryVoltage(Double batteryVoltage) {
        this.batteryVoltage = batteryVoltage;
        return this;
    }


    public String getDeviceSerialNumber() {
        return deviceSerialNumber;
    }

    public void setDeviceSerialNumber(String deviceSerialNumber) {
        this.deviceSerialNumber = deviceSerialNumber;
    }

    public String getDtc() {
        return dtc;
    }

    public VbusData setDtc(String dtc) {
        this.dtc = dtc;
        return this;
    }

    public DateTime getDtcTimestamp() {
        return null;
    }

    public VbusData setDtcTimestamp(DateTime dtcTimestamp) {
        this.dtcTimestamp = dtcTimestamp;
        return this;
    }

    public Double getEngineCoolantTempCelsius() {
        return engineCoolantTemperature;
    }

    public VbusData setEngCoolantTempCelsius(Double engineCoolantTemp) {
        engineCoolantTemperature = engineCoolantTemp;
        return this;
    }

    public Double getEngineOilPressurekPa() {
        return engineOilPressure;
    }

    public VbusData setEngOilPressurekPa(Double engineOilPressurekPa) {
        this.engineOilPressure = engineOilPressurekPa;
        return this;
    }

    public Double getEngineRpm() {
        return engineSpeedRpm;
    }

    public VbusData setEngRpm(Double engineRpm) {
        this.engineSpeedRpm = engineRpm;
        return this;
    }

    public DateTime getEngineRpmTimestamp() {
        return null;
    }

    public VbusData setEngineRpmTimestamp(DateTime engineRpmTimestamp) {
        this.engineRpmTimestamp = engineRpmTimestamp;
        return this;
    }

    public Double getEngineTotalHours() {
        return engineTotalHours;
    }

    public VbusData setEngineTotalHours(Double engineTotalHours) {
        this.engineTotalHours = engineTotalHours;
        return this;
    }

    public DateTime getEngineTotalHoursTimestamp() {
        return null;
    }

    public VbusData setEngineTotalHoursTimestamp(DateTime engTotalHoursTimestamp) {
        this.engineTotalHoursTimestamp = engTotalHoursTimestamp;
        return this;
    }

    public Double getFuelEconomy() {
        return fuelEconomy;
    }

    public VbusData setFuelEconomy(Double fuelEconomy) {
        this.fuelEconomy = fuelEconomy;
        return this;
    }

    public Double getFuelLevel() {
        return fuelLevel;
    }

    public VbusData setFuelLevel(Double fuelLevel) {
        this.fuelLevel = fuelLevel;
        return this;
    }

    public Double getFuelRate() {
        return fuelRate;
    }

    public VbusData setFuelRate(Double fuelRate) {
        this.fuelRate = fuelRate;
        return this;
    }

    public Double getFuelUsed() {
        return totalFuelUsed;
    }

    public VbusData setFuelUsed(Double fuelUsed) {
        this.totalFuelUsed = fuelUsed;
        return this;
    }

    public Double getHeading() {
        return heading;
    }

    public VbusData setHeading(Double heading) {
        this.heading = heading;
        return this;
    }

    public Double getHighResOdometerKm() {
        return highResOdometer;
    }

    public VbusData setHighResOdometerKm(Double highResOdometerKm) {
        this.highResOdometer = highResOdometerKm;
        return this;
    }

    public DateTime getHighResOdometerKmTimestamp() {
        return highResOdometerTimestamp;
    }

    public VbusData setHighResOdometerKmTimestamp(DateTime highResOdometerTimestamp) {
        this.highResOdometerTimestamp = highResOdometerTimestamp;
        return this;
    }

    public Double getHighResTripKm() {
        return highResTripKm;
    }

    public VbusData setHighResTripKm(Double highResTripKm) {
        this.highResTripKm = highResTripKm;
        return this;
    }

    public Boolean isIgnition() {
        return ignition;
    }

    public VbusData setIgnition(Boolean ignition) {
        this.ignition = ignition;
        return this;
    }

    public Double getLatitude() {
        return latitude;
    }

    public VbusData setLatitude(Double latitude) {
        this.latitude = latitude;
        return this;
    }

    public Double getLongitude() {
        return longitude;
    }

    public VbusData setLongitude(Double longitude) {
        this.longitude = longitude;
        return this;
    }

    public DateTime getLatlonTimestamp() {
        return null;
    }

    public VbusData setLatlonTimestamp(DateTime latlonTimestamp) {
        this.latlonTimestamp = latlonTimestamp;
        return this;
    }

    public String getManagerName() {
        return managerName;
    }

    public VbusData setManagerName(String managerName) {
        this.managerName = managerName;
        return this;
    }

    public Double getMilStatus() {
        return milStatus;
    }

    public VbusData setMilStatus(Double milStatus) {
        this.milStatus = milStatus;
        return this;
    }

    public Double getOdometerKm() {
        return odometer;
    }

    public VbusData setOdometerKm(Double odometerKm) {
        this.odometer = odometerKm;
        return this;
    }

    public DateTime getOdometerKmTimestamp() {
        return null;
    }

    public VbusData setOdometerKmTimestamp(DateTime odometerKm) {
        this.odometerKmTimestamp = odometerKm;
        return this;
    }

    public Integer getParkingBrake() {
        return parkingBrake;
    }

    public VbusData setParkingBrake(Integer parkingBrake) {
        this.parkingBrake = parkingBrake;
        return this;
    }

    public Boolean isPtoStatus() {
        return ptoStatus;
    }

    public VbusData setPtoStatus(Boolean ptoStatus) {
        this.ptoStatus = ptoStatus;
        return this;
    }

    public Boolean isSeatBelt() {
        return seatBelt;
    }

    public VbusData setSeatBelt(Boolean seatBelt) {

        this.seatBelt = seatBelt;
        return this;
    }

    public Double getThrottle() {
        return throttle;
    }

    public VbusData setThrottle(Double throttle) {
        this.throttle = throttle;
        return this;
    }

    public DateTime getTimestamp() {
        return timestamp;
    }

    public VbusData setTimestamp(DateTime timeStamp) {
        this.timestamp = timeStamp;
        return this;
    }

    public Integer getTransCurrentGear() {
        return transCurrentGear;
    }

    public VbusData setTransCurrentGear(Integer transCurrentGear) {
        this.transCurrentGear = transCurrentGear;
        return this;
    }

    public Double getTripEngineHours() {
        return tripEngineHours;
    }

    public VbusData setTripEngineHours(Double tripEngineHours) {
        this.tripEngineHours = tripEngineHours;
        return this;
    }

    public Double getTripFuelUsed() {
        return tripFuelUsed;
    }

    public VbusData setTripFuelUsedLiter(Double tripFuelUsed) {
        this.tripFuelUsed = tripFuelUsed;
        return this;
    }

    public Double getTripOdometerKm() {
        return tripOdometer;
    }

    public VbusData setTripOdometerKm(Double tripOdometerKm) {
        this.tripOdometer = tripOdometerKm;
        return this;
    }

    public Integer getTurnSignal() {
        return turnSignal;
    }

    public VbusData setTurnSignal(Integer turnSignal) {
        this.turnSignal = turnSignal;
        return this;
    }

    public Double getVehicleSpeedKph() {
        return vehicleSpeed;
    }

    public VbusData setVehicleSpeedKph(Double vehicleSpeedKph) {
        this.vehicleSpeed = vehicleSpeedKph;
        return this;
    }

    public DateTime getVehicleSpeedKphTimestamp() {
        return null;
    }

    public VbusData setVehicleSpeedKphTimestamp(DateTime vehicleSpeedKphTimestamp) {
        this.vehicleSpeedKphTimestamp = vehicleSpeedKphTimestamp;
        return this;
    }

    public String getVin() {
        return vin;
    }

    public VbusData setVin(String vin) {
        this.vin = vin;
        return this;
    }
}
